/**
 * 
 */
package de.ixxat.vci3;

/**
 * Base Interface for all VCI V3 Interfaces. The VCI interfaces provide access
 * to native driver resources. Because the Java garbage collector is only
 * designed to manage memory, but not native OS and driver resources the caller
 * is responsible to release this resources via calling
 * {@link IDisposable#Dispose()} when the object is no longer needed. Otherwise
 * native memory and resource leaks may occur.
 * 
 * @author Michael Ummenhofer
 * 
 */
public interface IDisposable
{

  /**
   * Releases the native interface resources. Call this function to free the
   * resources and set the object reference to <code>null</code> afterwards.
   * 
   * @throws Throwable
   */
  public void Dispose() throws Throwable;

}
