/**
 * 
 */
package de.ixxat.vci3;

import de.ixxat.Guid;
import de.ixxat.jni.CPointer;

/**
 * This value Class represents the VCI Device Information. The VCI Device
 * Information can be retrieved by calling {@link IVciDevice#GetDeviceInfo()}.
 * 
 * @author Michael Ummenhofer
 * 
 */
public class VciDeviceInfo
{
  /** Unique VCI Object Identifier */
  public long  m_qwVciObjectId          = 0;
  /** Device Class Identifier as GUID*/
  public Guid  m_oDeviceClass           = new Guid();

  /** Major driver version number */
  public short m_wDriverMajorVersion    = 0;
  /** Minor driver version number */
  public short m_wDriverMinorVersion    = 0;

  /** Major hardware version number */
  public short m_wHardwareMajorVersion  = 0;
  /** Minor hardware version number */
  public short m_wHardwareMinorVersion  = 0;

  /** unique hardware identifier as string "HW..." */
  public String m_strUniqueHardwareId   = "";
  /** unique hardware identifier as GUID */
  public Guid   m_oUniqueHardwareId     = new Guid();

  /** device description (e.g: "PC-I04-PCI") */
  public String m_strDescription        = "";
  /** device manufacturer (e.g: "IXXAT Automation") */
  public String m_strManufacturer       = "";

  /**
   * Creates a new VCI Device Information object.
   * @throws Throwable
   */
  protected VciDeviceInfo()
  {
  }

  /**
   * Creates a new VCI Device Information object.
   * 
   * @param pVciDeviceInfo
   *          CPointer to native memory which contains the VCIDEVICEINFO
   *          structure.
   * @throws Throwable
   */
  protected VciDeviceInfo(CPointer pVciDeviceInfo) throws Throwable
  {
    fromCPointer(pVciDeviceInfo);
  }

  /**
   * Updates a existing VCI Device Information object.
   * 
   * @param pVciDeviceInfo
   *          CPointer to native memory which contains the VCIDEVICEINFO
   *          structure.
   * @throws Throwable
   */
  protected void fromCPointer(CPointer pVciDeviceInfo) throws Throwable
  {
    String strTemp = "";
    
    // Update VCI Device Info
    m_qwVciObjectId         = pVciDeviceInfo.getLong(0);
    m_oDeviceClass.fromCPointer(pVciDeviceInfo, 8);
    m_wDriverMajorVersion   = pVciDeviceInfo.getShort(24);
    m_wDriverMinorVersion   = pVciDeviceInfo.getShort(26);
    m_wHardwareMajorVersion = pVciDeviceInfo.getShort(28);
    m_wHardwareMinorVersion = pVciDeviceInfo.getShort(30);
    
    // Check if unique HW ID as string is available
    m_strUniqueHardwareId = "";
    strTemp = pVciDeviceInfo.getString(32);
    if(strTemp.length() >=2 )
    {
      // Begins with HW
      if(((strTemp.toUpperCase().charAt(0)) == 'H') &&
         ((strTemp.toUpperCase().charAt(1)) == 'W'))
      {
        m_strUniqueHardwareId = strTemp;
        strTemp = null;
      }
    }

    m_oUniqueHardwareId.fromCPointer(pVciDeviceInfo, 32);
    m_strDescription        = pVciDeviceInfo.getString(48);
    m_strManufacturer       = pVciDeviceInfo.getString(176);
  }

  /**
   * Returns the VCI Device Information as string.
   * @return The VCI Device Information as string.
   */
  public String toString()
  {
    String strVciDeviceInfo = "";

    strVciDeviceInfo += "VCI Object ID: " + m_qwVciObjectId + ", ";
    strVciDeviceInfo += "VCI Device Class: " + m_oDeviceClass + ", ";
    strVciDeviceInfo += "Driver Version: " + m_wDriverMajorVersion + "." + m_wDriverMinorVersion + ", ";
    strVciDeviceInfo += "Hardware Version: " + m_wHardwareMajorVersion + "." + m_wHardwareMinorVersion + ", ";
    strVciDeviceInfo += "Unique HW ID: ";
    if(m_strUniqueHardwareId.length() > 0)
    {
      strVciDeviceInfo += "\"" + m_strUniqueHardwareId + "\" ";
    }
    strVciDeviceInfo += m_oUniqueHardwareId + ", ";
    strVciDeviceInfo += "Description: \"" + m_strDescription + "\", ";
    strVciDeviceInfo += "Manufacturer: \"" + m_strManufacturer + "\" ";

    return strVciDeviceInfo;
  }
}
