/**
 * 
 */
package de.ixxat.vci3.bal;

import de.ixxat.ZeroHex;
import de.ixxat.jni.CPointer;
import de.ixxat.vci3.VciDeviceCapabilities;

/**
 * This value Class represents the BAL Socket Info.
 * 
 * @author Michael Ummenhofer
 *
 */
public class BalSocketInfo
{
  /** VCI ID Object identifier */
  public long   m_qwObjectId        = 0;
  /** type of the bus socket */
  private short m_wBusCtrlType      = 0;   
  /** number of the bus socket */
  public short  m_wSocketNo         = 0;
  
  /** VCI BUS Type */
  public short    m_wBusType        = VciDeviceCapabilities.VCI_BUS_RES;
  /** VCI Controller Type */
  public short    m_wControllerType = VciDeviceCapabilities.VCI_CTRL_UNKNOWN;
  
  /**
   * Creates a new BAL Socket Info object.
   * @throws Throwable
   */
  public BalSocketInfo()
  {
  }

  /**
   * Creates a new BAL Socket Info object.
   * 
   * @param pBalSocketInfo
   *          CPointer to native memory which contains the BALSOCKETINFO
   *          structure.
   * @throws Throwable
   */
  public BalSocketInfo(CPointer pBalSocketInfo) throws Throwable
  {
    fromCPointer(pBalSocketInfo);
  }

  /**
   * Updates a existing BAL Socket Info object.
   * 
   * @param pBalSocketInfo
   *          CPointer to native memory which contains the BALSOCKETINFO
   *          structure.
   * @throws Throwable
   */
  public void fromCPointer(CPointer pBalSocketInfo) throws Throwable
  {
    // Update Socket Info
    m_qwObjectId    = pBalSocketInfo.getLong(0);
    m_wBusCtrlType  = pBalSocketInfo.getShort(8);
    m_wSocketNo     = pBalSocketInfo.getShort(10);
    
    // Extract Bus and Controller Type
    m_wBusType        = (short)((m_wBusCtrlType >> 8) & 0x00FF);
    m_wControllerType = (short)((m_wBusCtrlType >> 0) & 0x00FF);
  }

  /**
   * Returns the BAL Socket Info as string.
   * @return The BAL Socket Info as string.
   */
  public String toString()
  {
    String strBalSocketInfo = "";

    strBalSocketInfo += "Object ID: 0x" + ZeroHex.toZeroHexString(m_qwObjectId) + ", ";
    strBalSocketInfo += "Bus Type: " + m_wBusType + ", ";
    strBalSocketInfo += "Controller Type: " + m_wControllerType + ", ";
    strBalSocketInfo += "Socket No.: " + m_wSocketNo;

    return strBalSocketInfo;
  }

}
