/**
 * 
 */
package de.ixxat.vci3.bal.can;

import de.ixxat.ZeroHex;
import de.ixxat.jni.*;

/**
 * This value class represents the CAN Scheduler Status. The CAN Scheduler Status
 * can be retrieved by calling {@link ICanScheduler#GetStatus()}.
 * 
 * @author Michael Ummenhofer
 * 
 */
public class CanSchedulerStatus
{
  /** cyclic transmit task stopped */
  public static final byte CAN_CTXTSK_STAT_STOPPED  = 0x00;
  /** cyclic transmit task running */
  public static final byte CAN_CTXTSK_STAT_RUNNING  = 0x01;
  
  /** the message entry is empty */
  public static final byte CAN_CTXMSG_STAT_EMPTY    = 0x00;
  /** the processing of the message in progress */
  public static final byte CAN_CTXMSG_STAT_BUSY     = 0x01;
  /** the processing of the message has completed */
  public static final byte CAN_CTXMSG_STAT_DONE     = 0x02;
  
  /** status of cyclic transmit task see {@link #CAN_CTXTSK_STAT_STOPPED}*/
  public byte  m_bTaskStat    = 0;

  /** status of cyclic transmit task see {@link #CAN_CTXTSK_STAT_STOPPED}*/
  public byte  m_abMsgStat[]  = new byte[ICanScheduler.CAN_MAX_CTX_MSGS];

  /**
   * Creates a new CAN Channel Status object.
   * @throws Throwable
   */
  protected CanSchedulerStatus()
  {
  }
  
  /**
   * Creates a new CAN Channel Status object.
   * 
   * @param pCanSchedulerStatus
   *          CPointer to native memory which contains the CANSCHEDULERSTATUS
   *          structure.
   * @throws Throwable
   */
  protected CanSchedulerStatus(CPointer pCanSchedulerStatus) throws Throwable
  {
    fromCPointer(pCanSchedulerStatus);
  }
  
  /**
   * Updates a existing CAN Channel Status object.
   * 
   * @param pCanSchedulerStatus
   *          CPointer to native memory which contains the CANSCHEDULERSTATUS
   *          structure.
   * @throws Throwable
   */
  protected void fromCPointer(CPointer pCanSchedulerStatus) throws Throwable
  {
    // Update CAN Scheduler Status
    m_bTaskStat = pCanSchedulerStatus.getByte(0);
    pCanSchedulerStatus.copyOut(1, m_abMsgStat, 0, m_abMsgStat.length);
  }

  /**
   * Returns the CAN Channel Status as string.
   * @return The CAN Channel Status as string.
   */
  public String toString()
  {
    String strCanSchedulerStatus = "";
    
    strCanSchedulerStatus += "Task Status: 0x" + ZeroHex.toZeroHexString(m_bTaskStat) + ", ";
    strCanSchedulerStatus += "Message Status: ";
    
    for(int iIndex=0; iIndex < m_abMsgStat.length; iIndex++)
    {
      strCanSchedulerStatus += "Msg " + iIndex + ": 0x" + ZeroHex.toZeroHexString(m_abMsgStat[iIndex]);
      if((iIndex+1) != m_abMsgStat.length)
        strCanSchedulerStatus += ", ";
    }

    return strCanSchedulerStatus;
  }

}
