/**
 * 
 */
package de.ixxat.vci3.bal.lin;

import de.ixxat.jni.*;

/**
 * This value class represents the LIN monitor status. The LIN monitor status
 * can be retrieved by calling {@link ILinMonitor#GetStatus()}.
 * 
 * @author Michael Ummenhofer
 * 
 */
public class LinMonitorStatus
{
  /** current LIN line status */
  public LinLineStatus  m_oLinLineStatus  = new LinLineStatus();
  
  /**
   * TRUE if the channel is activated.
   * <p>
   * For {@link #toString()}: <code>"A"</code> means channel is activated
   */
  public boolean m_fActivated = false;

  /**
   * TRUE if receive FIFO overrun occurs
   * <p>
   * For {@link #toString()}: <code>"Q"</code> means receive FIFO overrun
   * occurred
   */
  public boolean m_fRxOverrun = false;
  
  /** receive FIFO load in percent (0..100) */
  public byte           m_bRxFifoLoad     = 0; 
  
  /**
   * Creates a new LIN Channel Status object.
   * @throws Throwable
   */
  protected LinMonitorStatus()
  {
  }
  
  /**
   * Creates a new LIN Channel Status object.
   * 
   * @param pLinChannelStatus
   *          CPointer to native memory which contains the LINCHANSTATUS
   *          structure.
   * @throws Throwable
   */
  protected LinMonitorStatus(CPointer pLinChannelStatus) throws Throwable
  {
    fromCPointer(pLinChannelStatus);
  }
  
  /**
   * Updates a existing LIN Channel Status object.
   * 
   * @param pLinChannelStatus
   *          CPointer to native memory which contains the LINCHANSTATUS
   *          structure.
   * @throws Throwable
   */
  protected void fromCPointer(CPointer pLinChannelStatus) throws Throwable
  {
    int dwActivated = 0;
    int dwRxOverrun = 0;

    // Update LIN Line Status
    m_oLinLineStatus.fromCPointer(pLinChannelStatus);
    
    // Update LIN Channel Status
    dwActivated   = pLinChannelStatus.getInt(8);
    dwRxOverrun   = pLinChannelStatus.getInt(12);
    m_bRxFifoLoad = pLinChannelStatus.getByte(16);
    
    if(dwActivated != 0)
      m_fActivated = true;
    else
      m_fActivated = false;
    
    if(dwRxOverrun != 0)
      m_fRxOverrun = true;
    else
      m_fRxOverrun = false;
  }

  /**
   * Returns the LIN Channel Status as string.
   * @return The LIN Channel Status as string.
   */
  public String toString()
  {
    String strLinChannelStatus = "";
    
    strLinChannelStatus += "RxFifoLoad: " + m_bRxFifoLoad + "%, ";
    strLinChannelStatus += "Flags: ";
    
    if(m_fActivated)
      strLinChannelStatus += "A";
    else
      strLinChannelStatus += " ";
    
    if(m_fRxOverrun)
      strLinChannelStatus += "O";
    else
      strLinChannelStatus += " ";

    return strLinChannelStatus;
  }

}
